#include <msp/core/getopt.h>
#include <msp/fs/utils.h>
#include <msp/strings/lexicalcast.h>
#include <msp/strings/regex.h>
#include "options.h"

using namespace std;
using namespace Msp;

Options::Options(int argc, char **argv):
	screen_w(1280),
	screen_h(960),
	fullscreen(false),
	debug(false),
	network(false),
	simulate(false),
	sim_speed(1.0f)
{
	string res;

	GetOpt getopt;
	getopt.add_option('r', "resolution",  res,         GetOpt::REQUIRED_ARG);
	getopt.add_option('f', "fullscreen",  fullscreen,  GetOpt::NO_ARG);
	getopt.add_option('g', "debug",       debug,       GetOpt::NO_ARG);
	getopt.add_option('d', "driver",      driver,      GetOpt::REQUIRED_ARG);
	getopt.add_option('s', "simulate",    simulate,    GetOpt::NO_ARG);
	getopt.add_option(     "sim-speed",   sim_speed,   GetOpt::REQUIRED_ARG);
	getopt.add_option('n', "network",     network,     GetOpt::NO_ARG);
	getopt.add_option(     "state",       state_fn,    GetOpt::REQUIRED_ARG);
	getopt.add_option(     "uistate",     uistate_fn,  GetOpt::REQUIRED_ARG);
	getopt.add_argument("layout", layout_fn, GetOpt::REQUIRED_ARG);
	getopt(argc, argv);

	if(!res.empty())
	{
		if(RegMatch m=Regex("([1-9][0-9]*)x([1-9][0-9]*)").match(res))
		{
			screen_w = lexical_cast<unsigned>(m[1].str);
			screen_h = lexical_cast<unsigned>(m[2].str);
		}
		else
			throw usage_error("Invalid resolution");
	}

	if(state_fn.empty())
		state_fn = FS::basepart(layout_fn)+".state";

	if(uistate_fn.empty())
		uistate_fn = FS::basepart(layout_fn)+".uistate";
}
