#include <msp/core/maputils.h>
#include <msp/core/raii.h>
#include <msp/gltk/button.h>
#include <msp/gltk/column.h>
#include "libr2c2/layout.h"
#include "libr2c2/route.h"
#include "libr2c2/trainrouter.h"
#include "engineer.h"
#include "routerpanel.h"

using namespace std;
using namespace Msp;
using namespace R2C2;

string route_name(const Route *const &route)
{
	return route ? route->get_name() : "(none)";
}

RouterPanel::RouterPanel(Engineer &e, Train &t):
	engineer(e),
	train(t),
	routes(&route_name),
	updating(false),
	goto_pick(false),
	goto_target(0),
	goto_highlight(0)
{
	Loader::WidgetMap widgets;
	DataFile::load(*this, "data/routerpanel.ui", widgets);

	lbl_route = dynamic_cast<GLtk::Label *>(get_item(widgets, "lbl_route"));
	drp_routes = dynamic_cast<GLtk::Dropdown *>(get_item(widgets, "drp_routes"));
	drp_routes->set_data(routes);
	drp_routes->signal_item_selected.connect(sigc::mem_fun(this, &RouterPanel::route_selected));

	dynamic_cast<GLtk::Button *>(get_item(widgets, "btn_goto"))->signal_clicked.connect(sigc::mem_fun(this, &RouterPanel::goto_clicked));

	routes.append(0);
	const set<Route *> &lroutes = train.get_layout().get_all<Route>();
	for(set<Route *>::const_iterator i=lroutes.begin(); i!=lroutes.end(); ++i)
		if(!(*i)->is_temporary())
			routes.append(*i);

	TrainRouter *router = train.get_ai_of_type<TrainRouter>();
	if(!router)
		router = new TrainRouter(train);
	update_route(router->get_route());

	train.signal_ai_event.connect(sigc::mem_fun(this, &RouterPanel::ai_event));
}

void RouterPanel::ai_event(TrainAI &, const TrainAI::Message &msg)
{
	if(msg.type=="route-changed")
		update_route(msg.value.value<const Route *>());
}

void RouterPanel::update_route(const Route *route)
{
	SetFlag setf(updating);
	if(route)
		lbl_route->set_text(route->get_name());
	else
		lbl_route->set_text("Free run");
	drp_routes->set_selected_index(routes.find(route));
}

void RouterPanel::route_selected(unsigned index)
{
	if(!updating)
	{
		const Route *route = routes.get(index);
		train.ai_message(TrainAI::Message("set-route", route));
	}
}

void RouterPanel::goto_clicked()
{
	goto_pick = true;
	goto_target = 0;
	signal_grab_pointer.emit();
}

void RouterPanel::button_press(int x, int y, unsigned btn)
{
	Panel::button_press(x, y, btn);

	if(goto_pick)
	{
		signal_ungrab_pointer.emit();
		goto_pick = false;

		delete goto_highlight;
		goto_highlight = 0;

		if(goto_target && btn==1)
			train.ai_message(TrainAI::Message("set-destination", static_cast<const TrackChain *>(goto_target)));
	}
}

void RouterPanel::pointer_motion(int x, int y)
{
	Panel::pointer_motion(x, y);
	
	if(goto_pick)
	{
		int rx = x;
		int ry = y;
		map_coords_to_ancestor(rx, ry, *find_ancestor<GLtk::Root>());
		Ray ray = engineer.get_main_view().create_ray(rx, ry);
		Track *track = engineer.get_layout().pick<Track>(ray);
		if(track && &track->get_block()!=goto_target)
		{
			goto_target = &track->get_block();
			delete goto_highlight;
			goto_highlight = new TrackChain3D(engineer.get_layout_3d(), *goto_target);
			goto_highlight->set_color(GL::Color(0.7));
			goto_highlight->set_layer(2);
		}
	}
}
