#include "clock.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Clock::Clock():
	rate(1),
	stopped(false)
{ }

void Clock::set_rate(float s)
{
	rate = s;
}

void Clock::set_current_time(const Time::TimeDelta &t)
{
	current_time = t;
	signal_discontinuity.emit();
}

void Clock::stop(bool s)
{
	stopped = s;
}

void Clock::tick(const Time::TimeDelta &dt)
{
	if(stopped)
		return;

	unsigned prev_minute = current_time/Time::min;
	current_time += dt*rate;
	if(current_time>=Time::day)
		current_time -= Time::day;
	unsigned minute = current_time/Time::min;
	if(minute!=prev_minute)
		signal_minute.emit();
}

void Clock::save(list<DataFile::Statement> &st) const
{
	st.push_back((DataFile::Statement("rate"), rate));
	st.push_back((DataFile::Statement("time"), current_time.raw()));
}


Clock::Loader::Loader(Clock &c):
	DataFile::ObjectLoader<Clock>(c)
{
	add("rate", &Clock::rate);
	add("time", &Loader::time);
}

void Clock::Loader::time(Time::RawTime t)
{
	obj.set_current_time(Time::TimeDelta(t));
}

} // namespace R2C2
