#ifndef LIBR2C2_TRACKCHAIN_H_
#define LIBR2C2_TRACKCHAIN_H_

#include <set>
#include <stdexcept>
#include <sigc++/trackable.h>
#include <msp/datafile/statement.h>
#include "trackiter.h"

namespace R2C2 {

class Layout;
class Object;

class bad_chain: public std::logic_error
{
public:
	bad_chain(const std::string &w): std::logic_error(w) { }
	virtual ~bad_chain() throw() { }
};


class TrackChain: public sigc::trackable
{
public:
	enum Direction
	{
		UNSPECIFIED,
		UP,
		DOWN
	};

	typedef std::set<Track *> TrackSet;

protected:
	enum Validity
	{
		VALID,
		UNLINKED,
		BAD_PATH,
		INCOMPATIBLE
	};

public:
	sigc::signal<void, const std::string &> signal_name_changed;
	sigc::signal<void, Track &> signal_track_added;
	sigc::signal<void, Track &> signal_track_removed;

protected:
	Layout &layout;
	std::string name;
	TrackSet tracks;
	TrackIter ends[2];

	TrackChain(Layout &);
public:
	virtual ~TrackChain() { }

	virtual void set_name(const std::string &);
	const std::string &get_name() const { return name; }

	void add_track(Track &);
	void add_tracks(const TrackSet &);
protected:
	virtual void on_track_added(Track &) { }
	virtual Validity check_validity(Track &) const;
	static void throw_bad_chain(Validity);
private:
	void update_ends(Track &);
public:
	const TrackSet &get_tracks() const { return tracks; }
	bool has_track(Track &) const;
	virtual TrackIter iter_for(Track &, Direction) const;
	TrackIter get_end(unsigned) const;
	virtual TrackIter get_end(Direction) const { return get_end(0); }
	bool is_loop() const;

private:
	void object_removed(Object &);

public:
	virtual Msp::DataFile::Statement save_reference() const = 0;
};


void operator<<(Msp::LexicalConverter &, TrackChain::Direction);
void operator>>(const Msp::LexicalConverter &, TrackChain::Direction &);

} // namespace R2C2

#endif
