#include <msp/core/maputils.h>
#include "client.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Client::Client(Catalogue &c):
	catalogue(c),
	socket(0),
	comm(0),
	event_disp(0),
	power(false),
	halt(false)
{ }

Client::~Client()
{
	delete comm;
	delete socket;
	for(map<unsigned, NetTrain *>::iterator i=trains.begin(); i!=trains.end(); ++i)
		delete i->second;
}

void Client::use_event_dispatcher(Msp::IO::EventDispatcher &ed)
{
	event_disp = &ed;
	if(socket)
		event_disp->add(*socket);
}

void Client::connect(const Net::SockAddr &addr)
{
	socket = new Net::StreamSocket(addr.get_family());
	socket->connect(addr);
	if(event_disp)
		event_disp->add(*socket);
	comm = new Net::Communicator(*socket, proto, *this);
}

void Client::set_power(bool p)
{
	DriverStatePacket pkt;
	pkt.power = p;
	pkt.halt = halt;
	send(pkt);
}

void Client::set_halt(bool h)
{
	DriverStatePacket pkt;
	pkt.power = power;
	pkt.halt = h;
	send(pkt);
}

NetTrain &Client::get_train(unsigned addr) const
{
	return *get_item(trains, addr);
}

void Client::receive(const DriverStatePacket &pkt)
{
	if(pkt.power!=power)
	{
		power = pkt.power;
		signal_power_changed.emit(power);
	}
	if(pkt.halt!=halt)
	{
		halt = pkt.halt;
		signal_halt_changed.emit(halt);
	}
}

void Client::receive(const EmergencyPacket &pkt)
{
	signal_emergency.emit(pkt.message);
}

void Client::receive(const TrainInfoPacket &pkt)
{
	NetTrain *train = new NetTrain(*this, pkt);
	trains[pkt.address] = train;
	signal_train_added.emit(*train);
}

void Client::receive(const TrainControlPacket &pkt)
{
	get_train(pkt.address).process_packet(pkt);
}

void Client::receive(const TrainFunctionPacket &pkt)
{
	get_train(pkt.address).process_packet(pkt);
}

void Client::receive(const TrainAIControlPacket &pkt)
{
	get_train(pkt.address).process_packet(pkt);
}

void Client::receive(const TrainStatusPacket &pkt)
{
	get_train(pkt.address).process_packet(pkt);
}

void Client::receive(const RouteInfoPacket &pkt)
{
	routes.push_back(pkt.name);
}

void Client::receive(const TrainRoutePacket &pkt)
{
	get_train(pkt.address).process_packet(pkt);
}

void Client::receive(const ErrorPacket &pkt)
{
	signal_error.emit(pkt.message);
}

} // namespace R2C2
